emacs-devel
[Top][All Lists]
Advanced

[Date Prev][Date Next][Thread Prev][Thread Next][Date Index][Thread Index]

Re: Incorrect font weight selected


From: Yuri D'Elia
Subject: Re: Incorrect font weight selected
Date: Wed, 05 Jan 2022 17:19:35 +0100
User-agent: mu4e 1.7.5; emacs 29.0.50

On Tue, Dec 21 2021, Eli Zaretskii wrote:
>> From: Yuri D'Elia <wavexx@thregr.org>
>> Cc: Dmitry Gutov <dgutov@yandex.ru>, monnier@iro.umontreal.ca,
>>  larsi@gnus.org, emacs-devel@gnu.org
>> Date: Tue, 21 Dec 2021 13:27:03 +0100
>>
>> The annoying fact is that font selection is called in so many contexts
>> it's just time consuming to drill down and/or set a good breakpoint
>> condition.
>
> I usually set a breakpoint in xfaces.c where the face attribute(s)
> is/are being changed, and then step into the functions it calls to
> select the font.  It's a bit slow, since you have a lot to step
> through, but it makes sure it's the right calls you are tracing.

After looking at this through a couple of times, I came up with the
attached patch. When changing the default face, the resulting font
name/pattern is stored as a string. Whenever a new frame is created, the
string is used to re-open the face again. I guess this is done to
support creating frames across different terminal types preserving the
closest available font.

This is done by calling font_open_by_name, which builds up a font spec
again from the stored string (which at this point _should_ be fully
specified), then calls the font_open_by_spec. The problem seems that
font_open_by_spec is _explicitly_ requesting a normal
weight/slant/width. So if multiple candidates were available while
enumerating fonts, the regular variant was always picked irregardless of
our preference.

This used to work before since only a single variant was generally
enumerated. In the patch, instead of just resetting/overriding the spec,
we just preset a normal variant if the spec is incomplete. I included
also the family/foundry in the spec, which can be used during selection
instead of blindly ignoring it.

This seems to work correctly for me now, however I'm only testing on
x11. font_open_by_name is not used in many places aside frame creation.
This change seems coherent to me looking at the other calls.

>From 1e9e66541553b2a439f8748b1ae12f398040d7e5 Mon Sep 17 00:00:00 2001
From: Yuri D'Elia <wavexx@thregr.org>
Date: Wed, 5 Jan 2022 17:11:23 +0100
Subject: [PATCH] Do not override the requested spec in font_open_by_spec

* src/font.c (font_open_by_spec): Do not override the requested spec
when opening a font. Only provide default values if the spec is
underspecified.

This ensures that, when loading fonts with multiple matching
alternatives, font_load_for_lface will pick the requested variant.
---
 src/font.c | 21 ++++++++++++++++-----
 1 file changed, 16 insertions(+), 5 deletions(-)

diff --git a/src/font.c b/src/font.c
index 58ff1a7981..1cb20f5796 100644
--- a/src/font.c
+++ b/src/font.c
@@ -3544,11 +3544,22 @@ font_open_by_spec (struct frame *f, Lisp_Object spec)
 {
   Lisp_Object attrs[LFACE_VECTOR_SIZE];
 
-  /* We set up the default font-related attributes of a face to prefer
-     a moderate font.  */
-  attrs[LFACE_FAMILY_INDEX] = attrs[LFACE_FOUNDRY_INDEX] = Qnil;
-  attrs[LFACE_SWIDTH_INDEX] = attrs[LFACE_WEIGHT_INDEX]
-    = attrs[LFACE_SLANT_INDEX] = Qnormal;
+  /* Filter according to spec.  */
+  attrs[LFACE_FAMILY_INDEX] = Ffont_get (spec, QCfamily);
+  attrs[LFACE_FOUNDRY_INDEX] = Ffont_get (spec, QCfoundry);
+  attrs[LFACE_SWIDTH_INDEX] = Ffont_get (spec, QCwidth);
+  attrs[LFACE_WEIGHT_INDEX] = Ffont_get (spec, QCweight);
+  attrs[LFACE_SLANT_INDEX] = Ffont_get (spec, QCslant);
+
+  /* We set up the fallback font-related attributes of a face to
+     prefer a moderate font.  */
+  if (NILP (attrs[LFACE_SWIDTH_INDEX]))
+    attrs[LFACE_SWIDTH_INDEX] = Qnormal;
+  if (NILP (attrs[LFACE_WEIGHT_INDEX]))
+    attrs[LFACE_WEIGHT_INDEX] = Qnormal;
+  if (NILP (attrs[LFACE_WEIGHT_INDEX]))
+    attrs[LFACE_WEIGHT_INDEX] = Qnormal;
+
 #ifndef HAVE_NS
   attrs[LFACE_HEIGHT_INDEX] = make_fixnum (120);
 #else
-- 
2.34.1


reply via email to

[Prev in Thread] Current Thread [Next in Thread]